# SPDX-License-Identifier: Apache-2.0

import time

from celery import Celery
from celery.result import AsyncResult
from cliff.command import Command
from loguru import logger
from redis import Redis
from osism import settings
from osism.tasks import Config


class Run(Command):
    def get_parser(self, prog_name):
        parser = super(Run, self).get_parser(prog_name)
        parser.add_argument(
            "--delay",
            default=1,
            type=int,
            help="Delay in second(s) between two task checks",
        )
        parser.add_argument(
            "--refresh",
            default=0,
            type=int,
            help="While waiting for all tasks refresh the list n times",
        )
        parser.add_argument(
            "--live",
            default=False,
            help="Show live output from a started task until it is finished",
            action="store_true",
        )
        parser.add_argument(
            "--format",
            default="log",
            help="Output type",
            const="log",
            nargs="?",
            choices=["script", "log"],
        ),
        parser.add_argument(
            "--output",
            default=False,
            help="Show output from a finished task",
            action="store_true",
        )
        parser.add_argument(
            "task_id", nargs="*", type=str, help="ID of tasks to wait for"
        )
        return parser

    def get_all_task_ids(self, i):
        task_ids = []
        for _, tasks in i.scheduled().items():
            for task in tasks:
                task_ids.append(task["id"])

        for _, tasks in i.active().items():
            for task in tasks:
                task_ids.append(task["id"])

        return sorted(task_ids)

    def take_action(self, parsed_args):
        delay = parsed_args.delay
        format = parsed_args.format
        live = parsed_args.live
        output = parsed_args.output
        refresh = parsed_args.refresh
        task_ids = sorted(parsed_args.task_id)

        do_refresh = False

        app = Celery("wait")
        app.config_from_object(Config)
        i = app.control.inspect()

        if not task_ids:
            logger.info("No task IDs specified, wait for all currently running tasks")
            task_ids = self.get_all_task_ids(i)
            do_refresh = True

        tmp_task_ids = []
        while task_ids or do_refresh:
            if task_ids:
                task_id = task_ids.pop()
                result = AsyncResult(f"{task_id}", app=app)

                if result.state == "PENDING":
                    q = i.query_task(f"{task_id}")
                    if not len([x for x in q.values() if len(x)]):
                        if format == "log":
                            logger.info(f"Task {task_id} is unavailable")
                        elif format == "script":
                            print(f"{task_id} = UNAVAILABLE")
                    else:
                        if format == "log":
                            logger.info(f"Task {task_id} is in state PENDING")
                        elif format == "script":
                            print(f"{task_id} = PENDING")

                        tmp_task_ids.insert(0, task_id)

                elif result.state == "SUCCESS":
                    if format == "log":
                        logger.info(f"Task {task_id} is in state SUCCESS")
                    elif format == "script":
                        print(f"{task_id} = SUCCESS")

                    if output:
                        print(result.get())

                elif result.state == "STARTED":
                    if format == "log":
                        logger.info(f"Task {task_id} is in state STARTED")
                    elif format == "script":
                        print(f"{task_id} = STARTED")

                    if live:
                        redis = Redis(
                            host=settings.REDIS_HOST,
                            port=settings.REDIS_PORT,
                            db=settings.REDIS_DB,
                            socket_keepalive=True,
                        )
                        redis.ping()

                        last_id = 0
                        while_True = True
                        while while_True:
                            data = redis.xread(
                                {str(task_id): last_id}, count=1, block=1000
                            )
                            if data:
                                messages = data[0]
                                for message_id, message in messages[1]:
                                    last_id = message_id.decode()
                                    message_type = message[b"type"].decode()
                                    message_content = message[b"content"].decode()

                                    logger.debug(
                                        f"Processing message {last_id} of type {message_type}"
                                    )
                                    redis.xdel(str(task_id), last_id)

                                    if message_type == "stdout":
                                        print(message_content, end="")
                                    elif message_type == "rc":
                                        rc = int(message_content)
                                    elif (
                                        message_type == "action"
                                        and message_content == "quit"
                                    ):
                                        redis.close()
                                        if len(task_ids) == 1:
                                            return rc
                                        else:
                                            while_True = False
                    else:
                        tmp_task_ids.insert(0, task_id)

                if not task_ids and tmp_task_ids:
                    logger.info(f"Wait {delay} second(s) until the next check")
                    time.sleep(delay)

                    if do_refresh:
                        task_ids = sorted(
                            list(set(self.get_all_task_ids(i) + tmp_task_ids))
                        )
                        tmp_task_ids = []
                    else:
                        task_ids = tmp_task_ids
            else:
                if refresh > 0:
                    refresh = refresh - 1
                    logger.info(
                        f"Wait {delay} second(s) until refresh of running tasks"
                    )
                    time.sleep(delay)
                    task_ids = self.get_all_task_ids(i)
                    tmp_task_ids = []
                else:
                    do_refresh = False



# SPDX-License-Identifier: Apache-2.0

import functools
import io
import subprocess
from threading import RLock

from celery import Celery
from celery.signals import worker_process_init
import kombu.utils
from loguru import logger
from pottery import Redlock
from redis import Redis

from osism import settings
from osism.tasks import Config


# https://github.com/celery/kombu/issues/1804
if not getattr(kombu.utils.cached_property, "lock", None):
    setattr(
        kombu.utils.cached_property,
        "lock",
        functools.cached_property(lambda _: RLock()),
    )
    # Must call __set_name__ here since this cached property is not defined in the context of a class
    # Refer to https://docs.python.org/3/reference/datamodel.html#object.__set_name__
    kombu.utils.cached_property.lock.__set_name__(kombu.utils.cached_property, "lock")

app = Celery("reconciler")
app.config_from_object(Config)

redis = None


@worker_process_init.connect
def celery_init_worker(**kwargs):
    global redis

    redis = Redis(
        host=settings.REDIS_HOST,
        port=settings.REDIS_PORT,
        db=settings.REDIS_DB,
        socket_keepalive=True,
    )
    redis.ping()


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    sender.add_periodic_task(
        settings.INVENTORY_RECONCILER_SCHEDULE, run_on_change.s(), expires=10
    )


@app.task(bind=True, name="osism.tasks.reconciler.run")
def run(self, publish=True):
    lock = Redlock(
        key="lock_osism_tasks_reconciler_run", masters={redis}, auto_release_time=60
    )

    if lock.acquire(timeout=20):
        logger.info("RUN /run.sh")
        p = subprocess.Popen(
            "/run.sh", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        for line in io.TextIOWrapper(p.stdout, encoding="utf-8"):
            if publish:
                redis.xadd(self.request.id, {"type": "stdout", "content": line})

        rc = p.wait(timeout=60)

        if publish:
            redis.xadd(self.request.id, {"type": "rc", "content": rc})
            redis.xadd(self.request.id, {"type": "action", "content": "quit"})

        lock.release()


@app.task(bind=True, name="osism.tasks.reconciler.run_on_change")
def run_on_change(self):
    lock = Redlock(
        key="lock_osism_tasks_reconciler_run_on_change",
        masters={redis},
        auto_release_time=60,
    )

    if lock.acquire(timeout=20):
        logger.info("RUN /run.sh")
        p = subprocess.Popen(
            "/run.sh", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        p.wait()

        lock.release()


@app.task(bind=True, name="osism.tasks.reconciler.sync_inventory_with_netbox")
def sync_inventory_with_netbox(self):
    lock = Redlock(
        key="lock_osism_tasks_reconciler_sync_inventory_with_netbox",
        masters={redis},
        auto_release_time=60,
    )

    if lock.acquire(timeout=20):
        p = subprocess.Popen(
            "/sync-inventory-with-netbox.sh",
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )

        for line in io.TextIOWrapper(p.stdout, encoding="utf-8"):
            # NOTE: use task_id or request_id in future
            redis.publish(
                "netbox-sync-inventory-with-netbox", {"type": "stdout", "content": line}
            )

        lock.release()

    # NOTE: use task_id or request_id in future
    redis.publish(
        "netbox-sync-inventory-with-netbox", {"type": "action", "content": "quit"}
    )



# SPDX-License-Identifier: Apache-2.0

from cliff.command import Command
from loguru import logger

from osism.tasks import ansible


class Maintenance(Command):
    def get_parser(self, prog_name):
        parser = super(Maintenance, self).get_parser(prog_name)
        parser.add_argument(
            "host",
            nargs=1,
            type=str,
            help="Host to be set to maintenance",
        )
        return parser

    def take_action(self, parsed_args):
        host = parsed_args.host[0]

        logger.info(f"Set maintenance state on host {host}")

        arguments = [
            "-e status=True",
            f"-l {host}",
        ]

        ansible.run.delay(
            "generic",
            "state-maintenance",
            arguments,
        )


class Bootstrap(Command):
    def get_parser(self, prog_name):
        parser = super(Bootstrap, self).get_parser(prog_name)
        parser.add_argument(
            "host",
            nargs=1,
            type=str,
            help="Host to be set to bootstrapped",
        )
        return parser

    def take_action(self, parsed_args):
        host = parsed_args.host[0]

        logger.info(f"Set bootstraped state on host {host}")

        arguments = [
            "-e status=True",
            f"-l {host}",
        ]

        ansible.run.delay(
            "generic",
            "state-bootstrap",
            arguments,
        )



# SPDX-License-Identifier: Apache-2.0

from loguru import logger
import git
from pottery import Redlock

from osism import utils


def for_device(name, parameters={}):
    device = utils.nb.dcim.devices.get(name=name)

    if (
        "device_type" not in device.custom_fields
        or device.custom_fields["device_type"] != "switch"
    ):
        return

    if "Managed by OSISM" not in [str(x) for x in device.tags]:
        return

    if "deployment_enabled" in device.custom_fields and not bool(
        device.custom_fields["deployment_enabled"]
    ):
        return

    if "deployment_type" not in device.custom_fields:
        return

    # Allow only one change per time
    lock = Redlock(
        key=f"lock_diff_{name}", masters={utils.redis}, auto_release_time=120
    )
    lock.acquire()

    logger.info(
        f"Diff configuration for device {device.name} with plugin {device.custom_fields['deployment_type']}"
    )

    deployment_type = device.custom_fields["deployment_type"]
    logger.error(
        f"Deployment type {deployment_type} for device {device.name} not supported"
    )
    current_configuration = None

    repo = git.Repo.init(path="/state")

    try:
        last_configuration = repo.git.show(
            f"{repo.head.commit.hexsha}:{device.name}.cfg.j2"
        )
    except git.exc.GitCommandError:
        last_configuration = None

    logger.error(
        f"Deployment type {deployment_type} for device {device.name} not supported"
    )

    lock.release()



# SPDX-License-Identifier: Apache-2.0

import functools
from threading import RLock

from celery import Celery
import kombu.utils

from osism import settings
from osism.tasks import Config, run_ansible_in_environment

# https://github.com/celery/kombu/issues/1804
if not getattr(kombu.utils.cached_property, "lock", None):
    setattr(
        kombu.utils.cached_property,
        "lock",
        functools.cached_property(lambda _: RLock()),
    )
    # Must call __set_name__ here since this cached property is not defined in the context of a class
    # Refer to https://docs.python.org/3/reference/datamodel.html#object.__set_name__
    kombu.utils.cached_property.lock.__set_name__(kombu.utils.cached_property, "lock")

app = Celery("ansible")
app.config_from_object(Config)


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    sender.add_periodic_task(
        settings.GATHER_FACTS_SCHEDULE, gather_facts.s(), expires=10
    )


@app.task(bind=True, name="osism.tasks.ansible.gather_facts")
def gather_facts(self, publish=True):
    return run_ansible_in_environment(
        self.request.id, "osism-ansible", "generic", "facts", [], publish, False
    )


@app.task(bind=True, name="osism.tasks.ansible.run")
def run(
    self,
    environment,
    playbook,
    arguments,
    publish=True,
    locking=False,
    auto_release_time=3600,
):
    return run_ansible_in_environment(
        self.request.id,
        "osism-ansible",
        environment,
        playbook,
        arguments,
        publish,
        locking,
        auto_release_time,
    )



# SPDX-License-Identifier: Apache-2.0

import argparse
import time

from cliff.command import Command
from loguru import logger

from osism.core.enums import VALIDATE_PLAYBOOKS
from osism.tasks import ansible, ceph, kolla
from osism.utils import redis


class Run(Command):
    def get_parser(self, prog_name):
        parser = super(Run, self).get_parser(prog_name)
        parser.add_argument(
            "validator",
            nargs=1,
            type=str,
            help="Validator to run",
            choices=VALIDATE_PLAYBOOKS.keys(),
        )
        parser.add_argument(
            "arguments", nargs=argparse.REMAINDER, help="Other arguments for Ansible"
        )
        parser.add_argument(
            "--format",
            default="log",
            help="Output type",
            const="log",
            nargs="?",
            choices=["script", "log"],
        ),
        parser.add_argument(
            "--environment",
            default=None,
            help="Environment",
            type=str,
        ),
        parser.add_argument(
            "--timeout",
            default=300,
            type=int,
            help="Timeout to end if there is no output",
        )
        parser.add_argument(
            "--no-wait",
            default=False,
            help="Do not wait until the validator run has been completed",
            action="store_true",
        )
        return parser

    def _handle_task(self, t, wait, format, timeout):
        rc = 0
        if wait:
            stoptime = time.time() + timeout
            last_id = 0
            while time.time() < stoptime:
                data = redis.xread(
                    {str(t.task_id): last_id}, count=1, block=(timeout * 1000)
                )
                if data:
                    stoptime = time.time() + timeout
                    messages = data[0]
                    for message_id, message in messages[1]:
                        last_id = message_id.decode()
                        message_type = message[b"type"].decode()
                        message_content = message[b"content"].decode()

                        logger.debug(
                            f"Processing message {last_id} of type {message_type}"
                        )
                        redis.xdel(str(t.task_id), message_id)

                        if message_type == "stdout":
                            print(message_content, end="")
                        elif message_type == "rc":
                            rc = int(message_content)
                        elif message_type == "action" and message_content == "quit":
                            redis.close()
                            return rc

        else:
            if format == "log":
                logger.info(
                    f"Task {t.task_id} is running in background. No more output. Check ARA for logs."
                )
            elif format == "script":
                print(f"{t.task_id}")

            return rc

    def take_action(self, parsed_args):
        arguments = parsed_args.arguments
        environment = parsed_args.environment
        validator = parsed_args.validator[0]
        format = parsed_args.format
        timeout = parsed_args.timeout
        wait = not parsed_args.no_wait

        runtime = VALIDATE_PLAYBOOKS[validator]["runtime"]

        if "playbook" in VALIDATE_PLAYBOOKS[validator]:
            playbook = VALIDATE_PLAYBOOKS[validator]["playbook"]
        else:
            playbook = f"validate-{validator}"

        if runtime == "ceph-ansible":
            if not environment:
                environment = "ceph"
            t = ceph.run.delay(environment, playbook, arguments)
        elif runtime == "kolla-ansible":
            arguments.append("-e kolla_action=config_validate")
            if not environment:
                environment = "kolla"
            t = kolla.run.delay(environment, playbook, arguments)
        else:
            environment = VALIDATE_PLAYBOOKS[validator]["environment"]
            t = ansible.run.delay(environment, playbook, arguments)

        rc = self._handle_task(t, wait, format, timeout)

        return rc



# SPDX-License-Identifier: Apache-2.0

import os


# Read secret from file
def read_secret(secret_name):
    try:
        f = open("/run/secrets/" + secret_name, "r", encoding="utf-8")
    except EnvironmentError:
        return ""
    else:
        with f:
            return f.readline().strip()


OPENSEARCH_ADDRESS = os.getenv("OPENSEARCH_ADDRESS")
OPENSEARCH_PROTOCOL = os.getenv("OPENSEARCH_PROTOCOL", "https")
OPENSEARCH_PORT = os.getenv("OPENSEARCH_PORT")

REDIS_HOST: str = os.getenv("REDIS_HOST", "redis")
REDIS_PORT: int = int(os.getenv("REDIS_PORT", "6379"))
REDIS_DB: int = int(os.getenv("REDIS_DB", "0"))


NETBOX_URL = os.getenv("NETBOX_API")
NETBOX_TOKEN = os.getenv("NETBOX_TOKEN", read_secret("NETBOX_TOKEN"))
IGNORE_SSL_ERRORS = os.getenv("IGNORE_SSL_ERRORS", "True") == "True"

BASE_PATH = os.getenv("BASE_PATH", "/devicetype-library/device-types/")
VENDORS = os.getenv("VENDORS", "").split()

# 43200 seconds = 12 hours
GATHER_FACTS_SCHEDULE = float(os.getenv("GATHER_FACTS_SCHEDULE", "43200.0"))
INVENTORY_RECONCILER_SCHEDULE = float(
    os.getenv("INVENTORY_RECONCILER_SCHEDULE", "600.0")
)



# SPDX-License-Identifier: Apache-2.0

from celery import Celery
from celery.signals import worker_process_init
import keystoneauth1
from loguru import logger
import openstack
from redis import Redis
import yaml

from osism import settings
from osism.tasks import Config

app = Celery("conductor")
app.config_from_object(Config)


configuration = {}
redis = None


@worker_process_init.connect
def celery_init_worker(**kwargs):
    global configuration
    global redis

    redis = Redis(
        host=settings.REDIS_HOST,
        port=settings.REDIS_PORT,
        db=settings.REDIS_DB,
        socket_keepalive=True,
    )
    redis.ping()

    # Parameters come from the environment, OS_*
    try:
        conn = openstack.connect()
    except keystoneauth1.exceptions.auth_plugins.MissingRequiredOptions:
        pass

    with open("/etc/conductor.yml") as fp:
        configuration = yaml.load(fp, Loader=yaml.SafeLoader)

        if not configuration:
            logger.warning(
                "The conductor configuration is empty. That's probably wrong"
            )
            return

        # Resolve all IDs in the conductor.yml
        if Config.enable_ironic in ["True", "true", "Yes", "yes"]:
            if "ironic_parameters" not in configuration:
                logger.error(
                    "ironic_parameters not found in the conductor configuration"
                )
                return

            # TODO: use osism.tasks.openstack in the future
            if "driver_info" in configuration["ironic_parameters"]:
                if "deploy_kernel" in configuration["ironic_parameters"]["driver_info"]:
                    result = conn.image.find_image(
                        configuration["ironic_parameters"]["driver_info"][
                            "deploy_kernel"
                        ]
                    )
                    configuration["ironic_parameters"]["driver_info"][
                        "deploy_kernel"
                    ] = result.id

                if (
                    "deploy_ramdisk"
                    in configuration["ironic_parameters"]["driver_info"]
                ):
                    result = conn.image.find_image(
                        configuration["ironic_parameters"]["driver_info"][
                            "deploy_ramdisk"
                        ]
                    )
                    configuration["ironic_parameters"]["driver_info"][
                        "deploy_ramdisk"
                    ] = result.id

                if (
                    "cleaning_network"
                    in configuration["ironic_parameters"]["driver_info"]
                ):
                    result = conn.network.find_network(
                        configuration["ironic_parameters"]["driver_info"][
                            "cleaning_network"
                        ]
                    )
                    configuration["ironic_parameters"]["driver_info"][
                        "cleaning_network"
                    ] = result.id

                if (
                    "provisioning_network"
                    in configuration["ironic_parameters"]["driver_info"]
                ):
                    result = conn.network.find_network(
                        configuration["ironic_parameters"]["driver_info"][
                            "provisioning_network"
                        ]
                    )
                    configuration["ironic_parameters"]["driver_info"][
                        "provisioning_network"
                    ] = result.id


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    pass


@app.task(bind=True, name="osism.tasks.conductor.get_ironic_parameters")
def get_ironic_parameters(self):
    if "ironic_parameters" in configuration:
        return configuration["ironic_parameters"]

    return {}



# SPDX-License-Identifier: Apache-2.0

import pynetbox
from redis import Redis
import urllib3

from osism import settings

redis = Redis(
    host=settings.REDIS_HOST,
    port=settings.REDIS_PORT,
    db=settings.REDIS_DB,
    socket_keepalive=True,
)
redis.ping()

if settings.NETBOX_URL and settings.NETBOX_TOKEN:
    nb = pynetbox.api(settings.NETBOX_URL, token=settings.NETBOX_TOKEN)

    if settings.IGNORE_SSL_ERRORS and nb:
        import requests

        urllib3.disable_warnings()
        session = requests.Session()
        session.verify = False
        nb.http_session = session

else:
    nb = None


# https://stackoverflow.com/questions/2361426/get-the-first-item-from-an-iterable-that-matches-a-condition
def first(iterable, condition=lambda x: True):
    """
    Returns the first item in the `iterable` that
    satisfies the `condition`.

    If the condition is not given, returns the first item of
    the iterable.

    Raises `StopIteration` if no item satysfing the condition is found.

    >>> first( (1,2,3), condition=lambda x: x % 2 == 0)
    2
    >>> first(range(3, 100))
    3
    >>> first( () )
    Traceback (most recent call last):
    ...
    StopIteration
    """

    return next(x for x in iterable if condition(x))



# SPDX-License-Identifier: Apache-2.0

import argparse
import json
import subprocess

from cliff.command import Command
from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
import requests

from osism import settings


class Ansible(Command):
    def get_parser(self, prog_name):
        parser = super(Ansible, self).get_parser(prog_name)
        parser.add_argument(
            "parameter",
            nargs=argparse.REMAINDER,
            type=str,
            help="Parameters to add (all paraemters of the ara command are possible)",
        )
        return parser

    def take_action(self, parsed_args):
        parameters = " ".join(parsed_args.parameter)
        subprocess.call(
            f"/usr/local/bin/ara {parameters}",
            shell=True,
        )


class Container(Command):
    def get_parser(self, prog_name):
        parser = super(Container, self).get_parser(prog_name)
        parser.add_argument("host", nargs=1, type=str, help="Hostname or address")
        parser.add_argument(
            "container", nargs=1, type=str, help="Name of the container"
        )
        parser.add_argument(
            "parameter",
            nargs=argparse.REMAINDER,
            type=str,
            help="Parameters to add (all paraemters of the docker logs command are possible)",
        )
        return parser

    def take_action(self, parsed_args):
        host = parsed_args.host[0]
        container_name = parsed_args.container[0]
        parameters = " ".join(parsed_args.parameter)

        ssh_command = f"docker logs {parameters} {container_name}"
        ssh_options = "-o StrictHostKeyChecking=no -o LogLevel=ERROR"

        # FIXME: use paramiko or something else more Pythonic + make operator user + key configurable
        subprocess.call(
            f"/usr/bin/ssh -i /ansible/secrets/id_rsa.operator {ssh_options} dragon@{host} {ssh_command}",
            shell=True,
        )


class File(Command):
    def get_parser(self, prog_name):
        parser = super(File, self).get_parser(prog_name)
        parser.add_argument("host", nargs=1, type=str, help="Hostname or address")
        return parser

    def take_action(self, parsed_args):
        print("NOT YET IMPLEMENTED")


class Opensearch(Command):
    def get_parser(self, prog_name):
        parser = super(Opensearch, self).get_parser(prog_name)
        parser.add_argument(
            "--verbose",
            default=False,
            help="Verbose output",
            action="store_true",
        )
        return parser

    def take_action(self, parsed_args):
        session = PromptSession(history=FileHistory("/tmp/.opensearch.history"))
        verbose = parsed_args.verbose

        while True:
            query = session.prompt(">>> ")
            if query in ["Exit", "exit", "EXIT"]:
                break

            result = requests.post(
                f"{settings.OPENSEARCH_PROTOCOL}://{settings.OPENSEARCH_ADDRESS}:{settings.OPENSEARCH_PORT}/_plugins/_sql?format=json",
                data=json.dumps({"query": query}),
                headers={"content-type": "application/json"},
                verify=False,
            )
            data = result.json()
            if "hits" in data:
                for hit in data["hits"]["hits"]:
                    source = hit["_source"]
                    if verbose:
                        if "timestamp" not in source:
                            source["timestamp"] = source["@timestamp"]

                        if "programname" in source:
                            print(
                                f"{source['timestamp']} | {source['Hostname']} | {source['programname']} | {source['Payload']}"
                            )
                        else:
                            print(
                                f"{source['timestamp']} | {source['Hostname']} | {source['Payload']}"
                            )
                    else:
                        print(source["Payload"])
            else:
                print(data)



# SPDX-License-Identifier: Apache-2.0

import subprocess
import time

from cliff.command import Command
from loguru import logger

from osism.tasks import reconciler
from osism.utils import redis


class Run(Command):
    def take_action(self, parsed_args):
        logger.info(
            "The osism reconciler command is deprecated and will be removed. Use osism service reconciler."
        )
        # NOTE: use python interface in the future, works for the moment
        p = subprocess.Popen(
            "celery -A osism.tasks.reconciler worker -n reconciler --loglevel=INFO -Q reconciler",
            shell=True,
        )
        p.wait()


class Sync(Command):
    def get_parser(self, prog_name):
        parser = super(Sync, self).get_parser(prog_name)
        parser.add_argument(
            "--no-wait",
            default=False,
            help="Do not wait until the sync has been completed",
            action="store_true",
        )
        parser.add_argument(
            "--task-timeout",
            default=3600,
            type=int,
            help="Timeout for a scheduled task that has not been executed yet",
        )
        return parser

    def take_action(self, parsed_args):
        wait = not parsed_args.no_wait
        task_timeout = parsed_args.task_timeout

        t = reconciler.run.delay(publish=wait)
        if wait:
            logger.info(
                f"Task {t.task_id} is running in background. Output coming soon."
            )
            rc = 0
            stoptime = time.time() + task_timeout
            last_id = 0
            while time.time() < stoptime:
                data = redis.xread(
                    {str(t.task_id): last_id}, count=1, block=(300 * 1000)
                )
                if data:
                    stoptime = time.time() + task_timeout
                    messages = data[0]
                    for message_id, message in messages[1]:
                        last_id = message_id.decode()
                        message_type = message[b"type"].decode()
                        message_content = message[b"content"].decode()

                        logger.debug(
                            f"Processing message {last_id} of type {message_type}"
                        )
                        redis.xdel(str(t.task_id), last_id)

                        if message_type == "stdout":
                            print(message_content, end="")
                        elif message_type == "rc":
                            rc = int(message_content)
                        elif message_type == "action" and message_content == "quit":
                            redis.close()
                            return rc
        else:
            logger.info(f"Task {t.task_id} is running in background. No more output.")
